// jshint maxerr:1000
/* ---------------------------------------------------------------------------------------------------------

    SCRIBBLEMAKES - BLOG PAGES WIDGET V1.0
    --------------------------------------
    Last updated: 30th August 2023
    
    --------------------------------------
    
    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
    
    --------------------------------------
    
    Welcome to the Blog Pages Widget (BPW)! This widget creates a page directory and connects your posts to
      each other, similar to a blog. To integrate into your website, save this .js file to it and follow the
      instructions in the 'more information' link (https://scribblemakes.com/code/development#sc_bpw). 
      You will need to fill in the required variables below.
      
    To include this correctly in your web page, you can copy this html code between your <body> tags:
    ---------------------------------------------------------------------------------------- Start HTML Code
    <!-- Add this to your <head> element OR CSS file to hide posts that are set in the future -->
    <style>
      #sc_bpw_postContainer {display:none;}
      #sc_bpw_postContainer.sc_bpw_postContainer_visible {display:block;}
    </style>
    
    <!-- Creates a list of the blog posts (pages) written in this script -->
    <div id="sc_bpw_postList">
      <noscript>Javascript disabled, blog posts cannot be retrieved</noscript>
    </div>
    
    <!-- The container that post information (like the title and date) are added to -->
    <div id="sc_bpw_postContainer">
      <noscript>Javascript disabled, post information cannot be retrieved</noscript>
      <p>Enter any HTML inside the &lt;div&gt; element to make your post. The post will be given a &lt;header&gt; automatically.</p>
    </div>
    
    <!-- Add this at the end of the page to call the script for the blog pages -->
    <script src="scWidget_BlogPages.js"></script>
    ------------------------------------------------------------------------------------------ End HTML Code
    
    --------------------------------------
    
    /‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾\  Looking for more tutorials, widgets, and plugins?       /‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾‾\  
   ｜  ❮‾ |\  /|  / ⎡‾‾  ｜  Get them on my personal website:                      ｜  ❮‾ |\  /|  / ⎡‾‾  ｜
   ｜  _❯ | \/ | /  ⎣__  ｜   https://scribblemakes.com/code                       ｜  _❯ | \/ | /  ⎣__  ｜  
    \___________________/                                                          \____________________/ 
    
    --------------------------------------
    
    For more information on this script:
     https://scribblemakes.com/code/development#sc_bpw
    
--------------------------------------------------------------------------------------------------------- */

// FILL IN ANY VARIABLES BELOW IF YOU WANT TO CHANGE THEM FROM THE DEFAULT

// Where you enter your new posts, each post is an array with its URL (no .html), Date + Hour, Title, Tags, and Preview image (optional)
// The URL is local, so don't include any '/'. Hours are in 24 hour time
// For example: ["PAGENAME", "YYYY-MM-DD-HH-MM", "TITLE", "TAG_1 TAG_2", "IMAGE.PNG"]
const sc_bpw_postArray = [
    ["post1", "2023", "Example Post 1", "blog_post 1st testing", "example.png"],
    ["post2", "2023-08", "Example Post 2", "blog_post 2nd showing_another_tag"],
    ["post3", "2023-08-26", "Example Post 3", "blog_post 3rd testing", "example.png"],
    ["post4", "2023-08-26-08", "Example Post 4", "blog_post 4th showing_another_tag"],
    ["post5", "2023-08-26-08-30", "Example Post 5", "blog_post 5th testing", "example.png"],
    ["install", "2023-08-26-20-30", "Installation Guide", "installation", "install.png"],
    ["post7", "9999", "Scheduled for 9999yr", "blog_post 7th showing_another_tag"],
    ["", "2000", "Current Page Post", "blog_post testing"]
  ];

const sc_bpw_postFolder = "posts"; // Filepath to your posts, i.e. which folder your posts are in
const sc_bpw_imageFolder = "posts/imgs";  // Filepath to your post preview / body images
const sc_bpw_imagePreviewPlaceholder = "placeholder.png";  // What image to use if the post doesn't have one
const sc_bpw_blogHomepage = "../archive.html";  // Local URL from post folder, OR absolute URL

// Setting the correct timezone for dates
const sc_bpw_timezone = +10;  // What your timezone is compared to UTC, for example: AEST is UTC+10

// Displaying HTML elements
const sc_bpw_settingHTML_directoryId = "sc_bpw_postList"; // Replaces this ID with the post links
const sc_bpw_settingHTML_postTitle = "sc_bpw_postTitle";  // Changes this title to post title
const sc_bpw_settingHTML_postContainer = "sc_bpw_postContainer";  // Adds post header and footer to this id

// Display Setup for links and post layouts
const sc_bpw_settingPostList_chronoOrder = "new";  // Sets which posts are first: "new" "old" "noOrder"
var sc_bpw_settingPostList_maxPosts = 5;  // How many posts show on one page list, set to 0 for infinite
const sc_bpw_settingPostList_navOldText = "Older Posts >>"; // Text for showing older posts on the post list
const sc_bpw_settingPostList_navHomeText = "Home";  // Text to return to your index.html, leave empty to ignore
const sc_bpw_settingPostList_navNewText = "<< Newer Posts"; // Text for showing newer posts on the post list

const sc_bpw_settingPostSchedule = true; // If true, hides posts with dates set in the future
const sc_bpw_settingPost_showTags = true; // If true, shows tags on the post page
const sc_bpw_settingPostList_showTags = true; // If true, shows tags on the post list

const sc_bpw_settingPost_navPosition = "bottom"; // Where post navigation goes: "head" "top" "bottom" "none"
  const sc_bpw_settingPost_navNewText = "<< Newer Post"; // Text for the newer post button
  const sc_bpw_settingPost_navBackText = "Back";  // Text for the 'return to post list page' button
  const sc_bpw_settingPost_navOldText = "Older Post >>"; // Text for the older post button

const sc_bpw_settingPostFormat_Date = "Posted on MML DDTH, YYYY at HHS:MINXM"; // Formats the date on posts
const sc_bpw_settingPostListFormat_Date = "Posted DD/MM/YY";  // Post list date format
  // Date format below, leave variable empty to hide date
  /*  MML = Long Month (January), MMS = Short Month (Jan), MM = Number Month (01)
      DDTH = Day with th (1st), DDN = Day name (Monday), DDS = Day name short (Mon), DD = Day number (01)
      YY = Short year (87), YYYY = Long year (1987)
      HH = Hour in 24hr time (0-23), HHS = Hour in 12hr time (1-12)
      MIN = Minutes (00)
      XM = Am / Pm  */

var sc_bpw_rssLink = ""; // Link to your RSS file, leave blank to ignore it


// ---------------------------------------------------------------------------------------------------------
// ---------------------- DO NOT EDIT PAST THIS POINT UNLESS PROFICIENT IN JAVASCRIPT ----------------------
// ---------------------------------------------------------------------------------------------------------
// Blog link list variables
var sc_bpw_objList = "";
var sc_bpw_postListArray = [];
var sc_bpw_postShowIndex = 0; // Where to start in the array, used for max-post limits
if (sc_bpw_settingPostList_maxPosts == 0) {
  sc_bpw_settingPostList_maxPosts = 999999999999999;  // Sets 0 to a large amount
}
// Tag inclusion and exclusion lists
var sc_bpw_includeTags = [];
var sc_bpw_excludeTags = [];

// Sort blog post array IF in chronological order
if (sc_bpw_settingPostList_chronoOrder == "new" || sc_bpw_settingPostList_chronoOrder == "old") {
  sc_bpw_postArray.sort(sc_bpw_SortDates);
}

// Date variables
var sc_bpw_date_monthLong = [
    "January", "February", "March", "April", "May", "June", "July", "August", "September", "October",
    "November", "December"
  ];

// RSS link
if (sc_bpw_rssLink != "") {
  sc_bpw_rssLink = "<a href='" + sc_bpw_rssLink + "'>RSS</a>";
}

// Checks if page is a blog post or has blog links
if (document.getElementById(sc_bpw_settingHTML_postContainer)) { // Has post container
  sc_bpw_PostSetup();
}
if (document.getElementById(sc_bpw_settingHTML_directoryId)) {  // Has blog link replacer
  sc_bpw_AddLinks();
}


// -----------------------------------
// -------- SORTING FUNCTIONS --------
// -----------------------------------
function sc_bpw_SortDates(sc_bpw_a, sc_bpw_b) {
  var sc_bpw_postDateTextA = sc_bpw_a[1].split("-"); // Date text
    // Replace any missing dates and times if not provided, a year is REQUIRED
    if (!sc_bpw_postDateTextA[1]) {sc_bpw_postDateTextA.push("01");}  // Month
    if (!sc_bpw_postDateTextA[2]) {sc_bpw_postDateTextA.push("01");}  // Day
    if (!sc_bpw_postDateTextA[3]) {sc_bpw_postDateTextA.push("00");}  // Hour
    if (!sc_bpw_postDateTextA[4]) {sc_bpw_postDateTextA.push("00");}  // Minute
  var sc_bpw_postDateA = new Date(Date.UTC(sc_bpw_postDateTextA[0], sc_bpw_postDateTextA[1]-1, sc_bpw_postDateTextA[2], sc_bpw_postDateTextA[3], sc_bpw_postDateTextA[4], 0));  // Date object
  
  var sc_bpw_postDateTextB = sc_bpw_b[1].split("-"); // Date text
    // Replace any missing dates and times if not provided, a year is REQUIRED
    if (!sc_bpw_postDateTextB[1]) {sc_bpw_postDateTextB.push("01");}  // Month
    if (!sc_bpw_postDateTextB[2]) {sc_bpw_postDateTextB.push("01");}  // Day
    if (!sc_bpw_postDateTextB[3]) {sc_bpw_postDateTextB.push("00");}  // Hour
    if (!sc_bpw_postDateTextB[4]) {sc_bpw_postDateTextB.push("00");}  // Minute
  var sc_bpw_postDateB = new Date(Date.UTC(sc_bpw_postDateTextB[0], sc_bpw_postDateTextB[1]-1, sc_bpw_postDateTextB[2], sc_bpw_postDateTextB[3], sc_bpw_postDateTextB[4], 0));  // Date object
  
  if (sc_bpw_settingPostList_chronoOrder == "old") {
    return sc_bpw_postDateA - sc_bpw_postDateB;
  } else if (sc_bpw_settingPostList_chronoOrder == "new") {
    return sc_bpw_postDateB - sc_bpw_postDateA;
  }
}


// ----------------------------------
// ------ ADD POST INFORMATION ------
// ----------------------------------
function sc_bpw_PostSetup() {  
  // Index of post information
  var sc_bpw_postContainer = document.getElementById(sc_bpw_settingHTML_postContainer);
  var sc_bpw_postIndex = -1;
  
  // Remove scheduled and hidden posts from being linked to
  if (sc_bpw_settingPostSchedule) {
    // Adds CSS to hide post just in case the CSS doesn't exist (still needed in noscript version)
    var sc_bpw_CSSContainerHider = document.createElement("style");
    document.head.appendChild(sc_bpw_CSSContainerHider);
    sc_bpw_CSSContainerHider.innerHTML = `
        #` + sc_bpw_settingHTML_postContainer + ` {display:none;}
        #` + sc_bpw_settingHTML_postContainer + `.sc_bpw_postContainer_visible {display:block;}
      `;
    for (sc_bpw_i = 0; sc_bpw_i < sc_bpw_postArray.length; sc_bpw_i++) {
      // Correct time and date to match the viewer's timezone
      var sc_bpw_postDateText = sc_bpw_postArray[sc_bpw_i][1].split("-"); // Date text
        // Replace any missing dates and times if not provided, a year is REQUIRED
        if (!sc_bpw_postDateText[1]) {sc_bpw_postDateText.push("01");}  // Month
        if (!sc_bpw_postDateText[2]) {sc_bpw_postDateText.push("01");}  // Day
        if (!sc_bpw_postDateText[3]) {sc_bpw_postDateText.push("00");}  // Hour
        if (!sc_bpw_postDateText[4]) {sc_bpw_postDateText.push("00");}  // Minute
      var sc_bpw_postDate = new Date(Date.UTC(sc_bpw_postDateText[0], sc_bpw_postDateText[1]-1, sc_bpw_postDateText[2], sc_bpw_postDateText[3], sc_bpw_postDateText[4], 0));  // Date object
      sc_bpw_postDate = new Date(sc_bpw_postDate - (sc_bpw_timezone * 60 * 60000)); // Correct for Timezone
      
      if (new Date() < sc_bpw_postDate) {
        sc_bpw_postArray.splice(sc_bpw_i, 1);
        sc_bpw_i--;
      }
    }
  }
  
  // Find post information
  for (sc_bpw_i = 0; sc_bpw_i < sc_bpw_postArray.length; sc_bpw_i++) {
    if (sc_bpw_postArray[sc_bpw_i][0].endsWith(".html")) {  // Correct user adding .html
      sc_bpw_postArray[sc_bpw_i][0] = sc_bpw_postArray[sc_bpw_i][0].split(".html")[0];
    }
    if (window.location.href.split("#")[0].endsWith(sc_bpw_postArray[sc_bpw_i][0]) || window.location.href.split("#")[0].endsWith(sc_bpw_postArray[sc_bpw_i][0] + ".html") || window.location.href.split("#")[0].endsWith(sc_bpw_postArray[sc_bpw_i][0] + ".htm")) {
      sc_bpw_postIndex = sc_bpw_i;
      break;
    }
  }
  
  // Check post has valid information and URL
  if (sc_bpw_postIndex != -1) { // If post URL is found
    var sc_bpw_postTitle = sc_bpw_postArray[sc_bpw_postIndex][2];
    var sc_bpw_postTags = sc_bpw_postArray[sc_bpw_postIndex][3].split(" ");
    
    // Change page's title if it matches the replacement variable
    if (document.title.toLowerCase() == sc_bpw_settingHTML_postTitle.toLowerCase()) {
      document.title = sc_bpw_postTitle;
    }
    
    // Correct time and date to match the viewer's timezone
    var sc_bpw_postDateText = sc_bpw_postArray[sc_bpw_postIndex][1].split("-"); // Date text
      // Replace any missing dates and times if not provided, a year is REQUIRED
      if (!sc_bpw_postDateText[1]) {sc_bpw_postDateText.push("01");}  // Month
      if (!sc_bpw_postDateText[2]) {sc_bpw_postDateText.push("01");}  // Day
      if (!sc_bpw_postDateText[3]) {sc_bpw_postDateText.push("00");}  // Hour
      if (!sc_bpw_postDateText[4]) {sc_bpw_postDateText.push("00");}  // Minute
      sc_bpw_postArray[sc_bpw_postIndex][1] = sc_bpw_postDateText.join("-");
    var sc_bpw_postDate = new Date(Date.UTC(sc_bpw_postDateText[0], sc_bpw_postDateText[1]-1, sc_bpw_postDateText[2], sc_bpw_postDateText[3], sc_bpw_postDateText[4], 0));  // Date object
    sc_bpw_postDate = new Date(sc_bpw_postDate - (sc_bpw_timezone * 60 * 60000)); // Correct for timezone
    
    // If post schedule's true, hide posts that are set in the future
    if ((sc_bpw_settingPostSchedule && new Date() >= sc_bpw_postDate) || !sc_bpw_settingPostSchedule) {
      sc_bpw_postContainer.classList.add("sc_bpw_postContainer_visible"); // Shows posts with the right css
    }
    
    // Create formatted date text
    var sc_bpw_postDateFormatted = sc_bpw_settingPostFormat_Date;
    sc_bpw_postDateFormatted = sc_bpw_FormatDate(sc_bpw_postDateFormatted, sc_bpw_postDateText, sc_bpw_postDate);
    sc_bpw_postDateFormatted = "<span class='sc_bpw_postDate'>" + sc_bpw_postDateFormatted + "</span>";
    
    // If tags are shown, set them up
    var sc_bpw_postTagsFormatted = "";
    if (sc_bpw_settingPost_showTags) {
      sc_bpw_postTagsFormatted = "<div class='sc_bpw_postTagsHolder'>";
      for (sc_bpw_i = 0; sc_bpw_i < sc_bpw_postTags.length; sc_bpw_i++) {
        sc_bpw_postTags[sc_bpw_i] = sc_bpw_postTags[sc_bpw_i].replaceAll("_"," ");
        sc_bpw_postTagsFormatted += "<span class='sc_bpw_postTags'>" + sc_bpw_postTags[sc_bpw_i] + "</span>";
      }
      sc_bpw_postTagsFormatted += "</div>";
    }
    
    // Append the title, date, and tags to the post as a header
    sc_bpw_postContainer.innerHTML = `
      <header class="sc_bpw_postHeader">
        <h1>` + sc_bpw_postTitle + `</h1>
        ` + sc_bpw_postDateFormatted + sc_bpw_postTagsFormatted + `
      </header>
    ` + sc_bpw_postContainer.innerHTML;
    
    // Set up post links to view other posts
    if (sc_bpw_settingPost_navPosition.toLowerCase() != "none") {
      var sc_bpw_navContainer = document.createElement("div");
      sc_bpw_navContainer.id = "sc_bpw_postNavContainer";
      if (sc_bpw_postIndex > 0) {
        sc_bpw_navContainer.innerHTML += `<a href="` + sc_bpw_postArray[sc_bpw_postIndex-1][0] + `.html" title="` + sc_bpw_postArray[sc_bpw_postIndex-1][2] + `">
            ` + sc_bpw_settingPost_navNewText + `
          </a>
        `;
      } else {sc_bpw_navContainer.innerHTML += sc_bpw_rssLink;}
      if (sc_bpw_blogHomepage != "") {
        sc_bpw_navContainer.innerHTML += "<a href='" + sc_bpw_blogHomepage + "' title='Back'>" + sc_bpw_settingPost_navBackText + "</a>";
      } else {sc_bpw_navContainer.innerHTML += sc_bpw_rssLink;}
      if (sc_bpw_postIndex < sc_bpw_postArray.length-1) {
        sc_bpw_navContainer.innerHTML += `<a href="` + sc_bpw_postArray[sc_bpw_postIndex+1][0] + `.html" title="` + sc_bpw_postArray[sc_bpw_postIndex+1][2] + `">
            ` + sc_bpw_settingPost_navOldText + `
          </a>
        `;
      } else {sc_bpw_navContainer.innerHTML += sc_bpw_rssLink;}
      // Append post links depending on navPosition setting
      switch (sc_bpw_settingPost_navPosition.toLowerCase()) {
        case "head":
          document.getElementsByClassName("sc_bpw_postHeader")[0].appendChild(sc_bpw_navContainer);
          break;
        case "top":
          document.getElementsByClassName("sc_bpw_postHeader")[0].after(sc_bpw_navContainer);
          break;
        case "bottom":
          sc_bpw_postContainer.appendChild(sc_bpw_navContainer);
          break;
      }
    }
    
  } else {  // If post URL is not found
    var sc_bpw_noPostNotif = document.createElement("p");
    sc_bpw_noPostNotif.innerText = "Post information not found";
    sc_bpw_postContainer.parentNode.insertBefore(sc_bpw_noPostNotif, sc_bpw_postContainer);
    window.location.href = "/not_found";
  }
}



// ---------------------------------
// ---- ADD BLOG POST LINK LIST ----
// ---------------------------------
function sc_bpw_AddLinks() {
  sc_bpw_objList = document.getElementById(sc_bpw_settingHTML_directoryId); // Fetch link object
  sc_bpw_objList.innerHTML = "";  // Clear container for new post links
  
  // Format post links
  for (sc_bpw_i = 0; sc_bpw_i < sc_bpw_postArray.length; sc_bpw_i++) {
    var sc_bpw_postURL = sc_bpw_postFolder + `/` + sc_bpw_postArray[sc_bpw_i][0];
    var sc_bpw_postTitle = sc_bpw_postArray[sc_bpw_i][2];
    var sc_bpw_postTags = sc_bpw_postArray[sc_bpw_i][3].split(" ");
    
    // Set preview image or give placeholder image
    var sc_bpw_postImg = sc_bpw_imageFolder + `/` + sc_bpw_imagePreviewPlaceholder;
    if (sc_bpw_postArray[sc_bpw_i][4]) {
      sc_bpw_postImg = sc_bpw_imageFolder + `/` + sc_bpw_postArray[sc_bpw_i][4];
    }
    
    // Correct time and date to match the viewer's timezone
    var sc_bpw_postDateText = sc_bpw_postArray[sc_bpw_i][1].split("-"); // Date text
      // Replace any missing dates and times if not provided, a year is REQUIRED
      if (!sc_bpw_postDateText[1]) {sc_bpw_postDateText.push("01");}  // Month
      if (!sc_bpw_postDateText[2]) {sc_bpw_postDateText.push("01");}  // Day
      if (!sc_bpw_postDateText[3]) {sc_bpw_postDateText.push("00");}  // Hour
      if (!sc_bpw_postDateText[4]) {sc_bpw_postDateText.push("00");}  // Minute
      sc_bpw_postArray[sc_bpw_i][1] = sc_bpw_postDateText.join("-");
    var sc_bpw_postDate = new Date(Date.UTC(sc_bpw_postDateText[0], sc_bpw_postDateText[1]-1, sc_bpw_postDateText[2], sc_bpw_postDateText[3], sc_bpw_postDateText[4], 0));  // Date object
    sc_bpw_postDate = new Date(sc_bpw_postDate - (sc_bpw_timezone * 60 * 60000)); // Correct for timezone
    var sc_bpw_postDateFormatted = sc_bpw_settingPostListFormat_Date; // Create formatted text
    
    sc_bpw_postDateFormatted = sc_bpw_FormatDate(sc_bpw_postDateFormatted, sc_bpw_postDateText, sc_bpw_postDate);
    
    sc_bpw_postDateFormatted = "<span class='sc_bpw_postDate'>" + sc_bpw_postDateFormatted + "</span>";
    
  
    // Create post link object
    var sc_bpw_postObj = `
      <a href="` + sc_bpw_postURL + `" title="` + sc_bpw_postTitle + `" class="sc_bpw_postLink">
        <img src="` + sc_bpw_postImg + `" alt="` + sc_bpw_postTitle + ` preview image">
        <b>` + sc_bpw_postTitle + `</b>
        ` + sc_bpw_postDateFormatted + `
      </a>
    `;
    
    sc_bpw_postListArray.push(sc_bpw_postObj);
  }
  
  sc_bpw_ShowLinks();
}


function sc_bpw_ShowLinks() {
  sc_bpw_objList.innerHTML = "";  // Clear container for new post links
  var sc_bpw_postArrayShow = [];  // Temporary container that lets some be deleted for tag views
  var sc_bpw_tagArray = [];
  var sc_bpw_tagArrayObj = "";  // Post List tag object
  
  // Remove any posts and tags that shouldn't be shown yet because of time
  for (sc_bpw_i = 0; sc_bpw_i < sc_bpw_postArray.length; sc_bpw_i++) {
    // Correct time and date to match the viewer's timezone
    var sc_bpw_postDateText = sc_bpw_postArray[sc_bpw_i][1].split("-"); // Date text
    var sc_bpw_postDate = new Date(Date.UTC(sc_bpw_postDateText[0], sc_bpw_postDateText[1]-1, sc_bpw_postDateText[2], sc_bpw_postDateText[3], sc_bpw_postDateText[4], 0));  // Date object
    sc_bpw_postDate = new Date(sc_bpw_postDate - (sc_bpw_timezone * 60 * 60000)); // Correct for timezone
    
    // Check if time lines up and add it to the show array
    if ((sc_bpw_settingPostSchedule && new Date() >= sc_bpw_postDate) || !sc_bpw_settingPostSchedule) {
      sc_bpw_postArrayShow.push([sc_bpw_i, sc_bpw_postArray[sc_bpw_i]]);
    
      // Get tags from post and add to tag array
      var sc_bpw_postTags = sc_bpw_postArray[sc_bpw_i][3].split(" ");
      for (sc_bpw_ii = 0; sc_bpw_ii < sc_bpw_postTags.length; sc_bpw_ii++) {
        sc_bpw_postTags[sc_bpw_ii] = sc_bpw_postTags[sc_bpw_ii].replaceAll("_", " ");
        sc_bpw_tagArray.push(sc_bpw_postTags[sc_bpw_ii]);
      }
    }
  }
      
  // Show tags if setting is true
  if (sc_bpw_settingPostList_showTags) {  
    // Organise tags
    sc_bpw_tagArray = sc_bpw_tagArray.sort().filter(function(sc_bpw_item, sc_bpw_pos, sc_bpw_array) {
        return !sc_bpw_pos || sc_bpw_item != sc_bpw_array[sc_bpw_pos - 1];
      });
    // Add to tag holder
    sc_bpw_tagArrayObj = document.createElement("div");
    sc_bpw_tagArrayObj.id = "sc_bpw_tagHolder";
    for (sc_bpw_i = 0; sc_bpw_i < sc_bpw_tagArray.length; sc_bpw_i++){
      var sc_bpw_tagObjectAdd = document.createElement("a");
        sc_bpw_tagObjectAdd.classList.add("sc_bpw_tagButton");
        sc_bpw_tagObjectAdd.tabIndex = "0";
        sc_bpw_tagObjectAdd.innerText = sc_bpw_tagArray[sc_bpw_i];
        // Give tag button the right css class
        if (sc_bpw_includeTags.indexOf(sc_bpw_tagObjectAdd.innerHTML) > -1) {
          sc_bpw_tagObjectAdd.classList.add("sc_bpw_tagButton_include");
        } else if (sc_bpw_excludeTags.indexOf(sc_bpw_tagObjectAdd.innerHTML) > -1) {
          sc_bpw_tagObjectAdd.classList.add("sc_bpw_tagButton_exclude");
        } else {
          sc_bpw_tagObjectAdd.classList.add("sc_bpw_tagButton_none");
        }
      sc_bpw_tagArrayObj.appendChild(sc_bpw_tagObjectAdd);
    }
    sc_bpw_objList.appendChild(sc_bpw_tagArrayObj);
  }
  
  // Remove any posts with the wrong tags
  for (sc_bpw_i = 0; sc_bpw_i < sc_bpw_postArrayShow.length; sc_bpw_i++) {
    let sc_bpw_postHasIncluded = true;
    let sc_bpw_postHasExcluded = false;
    for (sc_bpw_ii = 0; sc_bpw_ii < sc_bpw_includeTags.length; sc_bpw_ii++) { // Check post has EVERY tag
      if (!sc_bpw_postArrayShow[sc_bpw_i][1][3].includes(sc_bpw_includeTags[sc_bpw_ii].replaceAll(" ", "_"))) {
        sc_bpw_postHasIncluded = false;
      }
    }
    for (sc_bpw_ii = 0; sc_bpw_ii < sc_bpw_excludeTags.length; sc_bpw_ii++) { // Check post has EVERY tag
      if (sc_bpw_postArrayShow[sc_bpw_i][1][3].includes(sc_bpw_excludeTags[sc_bpw_ii].replaceAll(" ", "_"))) {
        sc_bpw_postHasExcluded = true;
      }
    }
    
    // Remove from the array
    if (!sc_bpw_postHasIncluded || sc_bpw_postHasExcluded) {
      sc_bpw_postArrayShow.splice(sc_bpw_i, 1);
      sc_bpw_i--;
    }
  }
    
  // Show links if they're in the max-post range
  for (sc_bpw_i = sc_bpw_postShowIndex; sc_bpw_i < sc_bpw_postArrayShow.length && sc_bpw_i < (sc_bpw_settingPostList_maxPosts + sc_bpw_postShowIndex); sc_bpw_i++) {
    
    // Add post to list
    sc_bpw_objList.innerHTML += sc_bpw_postListArray[sc_bpw_postArrayShow[sc_bpw_i][0]];
    //sc_bpw_objList.innerHTML += sc_bpw_postArrayShow[sc_bpw_i];
  }
  
  // Add 'new posts' and 'old posts' buttons if post amount is more than the max limit
  if (sc_bpw_settingPostList_maxPosts < sc_bpw_postArrayShow.length) {
    var sc_bpw_listButtons = document.createElement("div"); // Button box
    sc_bpw_listButtons.id = "sc_bpw_postListNav";
    sc_bpw_objList.appendChild(sc_bpw_listButtons);
      var sc_bpw_listButtonsContainer = document.createElement("span"); // Contains buttons in button box
      sc_bpw_listButtonsContainer.id = "sc_bpw_postListNavButtons";
      sc_bpw_listButtons.appendChild(sc_bpw_listButtonsContainer);
    
      // Add button element for new posts, home, and old posts
      var sc_bpw_listNewButton = document.createElement("button");
      sc_bpw_listButtonsContainer.appendChild(sc_bpw_listNewButton);
      sc_bpw_listNewButton.innerText = sc_bpw_settingPostList_navNewText;
      sc_bpw_listNewButton.tabIndex = 0;
      if (sc_bpw_settingPostList_navHomeText != "") {
        var sc_bpw_listHomeButton = document.createElement("button");
        sc_bpw_listButtonsContainer.appendChild(sc_bpw_listHomeButton);
        sc_bpw_listHomeButton.innerText = sc_bpw_settingPostList_navHomeText;
        sc_bpw_listHomeButton.tabIndex = 0;
      }
      var sc_bpw_listOldButton = document.createElement("button");
      sc_bpw_listButtonsContainer.appendChild(sc_bpw_listOldButton);
      sc_bpw_listOldButton.innerText = sc_bpw_settingPostList_navOldText;
      sc_bpw_listOldButton.tabIndex = 0;
    
    // Add new posts, home, and old posts listeners
    if (sc_bpw_postShowIndex > 0) {
      sc_bpw_listNewButton.addEventListener("click", function() {
        sc_bpw_postShowIndex -= sc_bpw_settingPostList_maxPosts;
        sc_bpw_AddLinks();
      });
      sc_bpw_listNewButton.addEventListener('keydown',function(e) {
        if (e.keyCode == 13){
          sc_bpw_postShowIndex -= sc_bpw_settingPostList_maxPosts;
          sc_bpw_AddLinks();
        }
      });
    } else {sc_bpw_listNewButton.disabled = true;}
    if (sc_bpw_settingPostList_navHomeText != "") {
      sc_bpw_listHomeButton.addEventListener("click", function() {
          window.location.href = "/";
      });
      sc_bpw_listHomeButton.addEventListener('keydown',function(e) {
        if (e.keyCode == 13){
          window.location.href = "/";
        }
      });
    }
    if (sc_bpw_postShowIndex + sc_bpw_settingPostList_maxPosts < sc_bpw_postArrayShow.length) {
      sc_bpw_listOldButton.addEventListener("click", function() {
        sc_bpw_postShowIndex += sc_bpw_settingPostList_maxPosts;
        sc_bpw_AddLinks();
      });
      sc_bpw_listOldButton.addEventListener('keydown',function(e) {
        if (e.keyCode == 13){
          sc_bpw_postShowIndex += sc_bpw_settingPostList_maxPosts;
          sc_bpw_AddLinks();
        }
      });
    } else {sc_bpw_listOldButton.disabled = true;}
  }
  
  // Add listen events to tag buttons
    // This took hours out of me, reminder: add events /AFTER/ HTML is finished being edited
  for (sc_bpw_i = 0; sc_bpw_i < document.getElementById("sc_bpw_tagHolder").children.length; sc_bpw_i++) {
    document.getElementById("sc_bpw_tagHolder").children[sc_bpw_i].addEventListener("click", function(e) {
      sc_bpw_TagFilter(this);
    });
    document.getElementById("sc_bpw_tagHolder").children[sc_bpw_i].addEventListener("keydown", function(e) {
      if (e.key == "Enter") {
        sc_bpw_TagFilter(this);
      }
    });
  }
}

// ---------------------------------
// ---- TAG FILTERING FUNCTIONS ----
// ---------------------------------
function sc_bpw_TagFilter(sc_bpw_tagObj_toggled) {
  let sc_bpw_tagName = sc_bpw_tagObj_toggled.innerHTML;
  if (sc_bpw_tagObj_toggled.classList.contains("sc_bpw_tagButton_none")) {  // Swap to include
    sc_bpw_tagObj_toggled.classList.remove("sc_bpw_tagButton_none");
    sc_bpw_tagObj_toggled.classList.add("sc_bpw_tagButton_include");
    if (sc_bpw_excludeTags.indexOf(sc_bpw_tagName) > -1) {  // Remove from excluded tags
      sc_bpw_excludeTags.splice(sc_bpw_excludeTags.indexOf(sc_bpw_tagName), 1);
    }
    sc_bpw_includeTags.push(sc_bpw_tagName);
  } 
  
  else if (sc_bpw_tagObj_toggled.classList.contains("sc_bpw_tagButton_include")) {  // Swap to exclude
    sc_bpw_tagObj_toggled.classList.remove("sc_bpw_tagButton_include");
    sc_bpw_tagObj_toggled.classList.add("sc_bpw_tagButton_exclude");
    if (sc_bpw_includeTags.indexOf(sc_bpw_tagName) > -1) {  // Remove from included tags
      sc_bpw_includeTags.splice(sc_bpw_includeTags.indexOf(sc_bpw_tagName), 1);
    }
    sc_bpw_excludeTags.push(sc_bpw_tagName);
  } 
  
  else if (sc_bpw_tagObj_toggled.classList.contains("sc_bpw_tagButton_exclude")) {  // Swap to no effect
    sc_bpw_tagObj_toggled.classList.remove("sc_bpw_tagButton_exclude");
    sc_bpw_tagObj_toggled.classList.add("sc_bpw_tagButton_none");
    if (sc_bpw_excludeTags.indexOf(sc_bpw_tagName) > -1) {  // Remove from excluded tags
      sc_bpw_excludeTags.splice(sc_bpw_excludeTags.indexOf(sc_bpw_tagName), 1);
    }
    if (sc_bpw_includeTags.indexOf(sc_bpw_tagName) > -1) {  // Remove from included tags
      sc_bpw_includeTags.splice(sc_bpw_includeTags.indexOf(sc_bpw_tagName), 1);
    }
  }
  
  // Reset displaying posts
  sc_bpw_postShowIndex = 0;
  sc_bpw_ShowLinks();
}


// ---------------------------------
// ---- FORMAT DATES FOR READER ----
// ---------------------------------
function sc_bpw_FormatDate(sc_bpw_postDateFormatted, sc_bpw_postDateText, sc_bpw_postDate) {
    // Find day's suffix
    var sc_bpw_postDateFormat_dayEnd = "th";
    if (10 <= parseInt(sc_bpw_postDateText[2]) && parseInt(sc_bpw_postDateText[2]) <= 19) { // Format Day
      sc_bpw_postDateFormat_dayEnd = "th";
    } else if (sc_bpw_postDateText[2].endsWith("1")) {
      sc_bpw_postDateFormat_dayEnd = "st";
    } else if (sc_bpw_postDateText[2].endsWith("2")) {
      sc_bpw_postDateFormat_dayEnd = "nd";
    } else if (sc_bpw_postDateText[2].endsWith("3")) {
      sc_bpw_postDateFormat_dayEnd = "rd";
    }
    
    // Format time
    var sc_bpw_postDateFormat_timeSuffix = "pm";
    var sc_bpw_postDateFormat_hourTwelve = sc_bpw_postDateText[3];
    if (parseInt(sc_bpw_postDateText[3]) == 0) {  
      sc_bpw_postDateFormat_hourTwelve = "12";
      sc_bpw_postDateFormat_timeSuffix = "am";
    } else if (parseInt(sc_bpw_postDateText[3]) > 11) {
      sc_bpw_postDateFormat_hourTwelve = (parseInt(sc_bpw_postDateText[3]) - 12);
    } else {
      sc_bpw_postDateFormat_timeSuffix = "am";
    }
    
    // Replace Day in date formatting
    sc_bpw_postDateFormatted = sc_bpw_postDateFormatted.replaceAll("DDTH",parseInt(sc_bpw_postDateText[2]) + sc_bpw_postDateFormat_dayEnd);
    sc_bpw_postDateFormatted = sc_bpw_postDateFormatted.replaceAll("DDN",sc_bpw_postDate.toLocaleString('en-AU', {weekday: 'long'}));
    sc_bpw_postDateFormatted = sc_bpw_postDateFormatted.replaceAll("DDS",sc_bpw_postDate.toLocaleString('en-AU', {weekday: 'short'}));
    sc_bpw_postDateFormatted = sc_bpw_postDateFormatted.replaceAll("DD",parseInt(sc_bpw_postDateText[2]));
    
    // Replace Month in date formatting
    sc_bpw_postDateFormatted = sc_bpw_postDateFormatted.replaceAll("MML",sc_bpw_date_monthLong[parseInt(sc_bpw_postDateText[1]) - 1]);
    sc_bpw_postDateFormatted = sc_bpw_postDateFormatted.replaceAll("MMS",sc_bpw_date_monthLong[parseInt(sc_bpw_postDateText[1]) - 1].substr(0,3));
    sc_bpw_postDateFormatted = sc_bpw_postDateFormatted.replaceAll("MM",sc_bpw_postDateText[1]);
    
    // Replace Year in date formatting
    sc_bpw_postDateFormatted = sc_bpw_postDateFormatted.replaceAll("YYYY",parseInt(sc_bpw_postDateText[0]));
    sc_bpw_postDateFormatted = sc_bpw_postDateFormatted.replaceAll("YY",parseInt(sc_bpw_postDateText[0].substr(sc_bpw_postDateText[0].length-2)));
    
    // Replace Hour in date formatting
    sc_bpw_postDateFormatted = sc_bpw_postDateFormatted.replaceAll("HHS",sc_bpw_postDateFormat_hourTwelve);
    sc_bpw_postDateFormatted = sc_bpw_postDateFormatted.replaceAll("HH",parseInt(sc_bpw_postDateText[3]));
    
    // Replace Minute and Suffix in date formatting
    sc_bpw_postDateFormatted = sc_bpw_postDateFormatted.replaceAll("MIN",sc_bpw_postDateText[4]);
    sc_bpw_postDateFormatted = sc_bpw_postDateFormatted.replaceAll("XM",sc_bpw_postDateFormat_timeSuffix);
    
    // Return value
    return sc_bpw_postDateFormatted;
}
